# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('admin_page_lock', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='databasepagelockmodel',
            options={'ordering': ('locked_at',), 'verbose_name': 'Page Lock', 'verbose_name_plural': 'Page Locks'},
        ),
        migrations.AlterField(
            model_name='databasepagelockmodel',
            name='locked_at',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='databasepagelockmodel',
            name='locked_out',
            field=models.DateTimeField(db_index=True),
        ),
    ]



from __future__ import unicode_literals

import datetime
import logging

from django.middleware.csrf import get_token
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from admin_page_lock.settings import (
    DISABLE_CRSF_TOKEN,
    HOMEPAGE,
    TIMEOUT
)


class PageLockHandler(object):
    """
    Provides logic for all API using different models that have same
    API defined.
    """
    HANDLER_NAME = 'Page-Lock-Handler'  # Used for logging messages.

    def __init__(self, req, model_class, *args, **kwargs):
        self.model_class = model_class
        self.page_settings = model_class.get_page_settings(req)

    def _get_now(self):
        return timezone.now()

    def _get_lock_settings(self):
        # TODO(vstefka) this settings can be specific for each page defined in
        # settings somehow or defined and stored in database. Right now, the
        # settings is same for all pages!
        lock_settings = {
            'csrf_token': get_token(self.page_settings['req']) if not DISABLE_CRSF_TOKEN else '',  # noqa: E501
            'homepage': HOMEPAGE,
            'messages': self.page_settings['messages'],
            'user_reference': self.page_settings['user_reference'],
        }

        return lock_settings

    def _log_message(self, message, status='debug'):
        # Log handler messages.
        loger_function = getattr(logging, status)
        page_url = self.page_settings['page_url']
        user_reference = self.page_settings['user_reference']

        loger_function(
            _('Handler: "{}"\nURL: "{}"\nUser: "{}"\nMessage: "{}"\n').format(
                self.HANDLER_NAME,
                page_url if page_url else '',
                user_reference if user_reference else '',
                message
            )
        )

    def close_page_connection(self, req, *args, **kwargs):
        """
        Closes page connection. Remove data related to the current page
        from storage or set it up to non-active only if same user is locking
        current page.
        """
        self._log_message(_('Close page connection.'))
        # Get data from storage.
        data = self.model_class.get_data(self.page_settings)

        # Deactivate lock if user is locking current page.
        if (
            data is not None and
            data['user_reference'] == self.page_settings['user_reference']
        ):
            self.model_class.deactivate(self.page_settings)

        return {'is_locked': False}

    def get_page_info(self, req, *args, **kwargs):
        """
        Returns page information:
         + is_locked            whether the page is locked (True/False);
         + locked_by            `user_reference` that locked the page;
         + page_lock_settings   returns general page lock settings;
         + reconnect_in         time interval when page might be available [s].
        """
        self._log_message(_('Get page info.'))
        response_data = {
            'page_lock_settings': self._get_lock_settings()
        }
        # Get data from storage.
        data = self.model_class.get_data(self.page_settings)

        # 1. No user is locking the page.
        if data is None:
            is_locked = False
            locked_by = None
            reconnect_in = 0
            reconnected = False
        # 2. Page is locked by current user or by another user.
        else:
            is_locked = True
            locked_by = data['user_reference']
            locked_out = data['locked_out']
            now = self._get_now()
            dtime = locked_out - now

            reconnect_in = 0
            if dtime.seconds < TIMEOUT:
                reconnect_in = dtime.seconds

            # Note: parameter `reconnected` is not used right now.
            reconnected = False
            if self.page_settings['user_reference'] == data['user_reference']:
                reconnected = True

        response_data.update({
            'is_locked': is_locked,
            'locked_by': locked_by,
            'reconnect_in': reconnect_in,
            'reconnected': reconnected
        })

        return response_data

    def open_page_connection(self, req, *args, **kwargs):
        """
        Opens/Reopens page connection and returns:
         + is_locked            whether page connection is locked (True/False);
         + locked_by            user locking current page;
         + page_lock_settings   current lock page configuratino;
         + reconnected          page is locked by same user;
         + reconnect_in         number of seconds when page might be available.
        """
        self._log_message(_('Open/Reopen page connection.'))
        response_data = {
            'page_lock_settings': self._get_lock_settings()
        }
        # Get data from storage.
        data = self.model_class.get_data(self.page_settings)

        # 1. Page is not locked and is going to be lock by current user.
        if data is None:
            is_locked = True
            locked_by = self.page_settings['user_reference']
            reconnected = False
            reconnect_in = TIMEOUT

            locked_at = self._get_now()
            locked_out = locked_at + datetime.timedelta(
                seconds=TIMEOUT)

            self.model_class.set_data(
                self.page_settings,
                {
                    'locked_at': locked_at,
                    'locked_out': locked_out,
                    'user_reference': locked_by
                }
            )

        # 2. Page is locked by same user.
        elif (
            data is not None and
            self.page_settings['user_reference'] == data['user_reference']
        ):
            is_locked = True
            locked_by = data['user_reference']
            reconnected = True
            reconnect_in = TIMEOUT

            # Deactivate previous data.
            self.model_class.deactivate(self.page_settings)

            # Store new data in storage.
            locked_at = self._get_now()
            locked_out = locked_at + datetime.timedelta(
                seconds=TIMEOUT)

            self.model_class.set_data(
                self.page_settings,
                {
                    'locked_at': locked_at,
                    'locked_out': locked_out,
                    'user_reference': locked_by,
                }
            )
        # 3. Page is locked by another user.
        elif (
            data is not None and
            self.page_settings['user_reference'] != data['user_reference']
        ):
            is_locked = True
            locked_by = data['user_reference']
            reconnected = False
            reconnect_in = TIMEOUT
            locked_out = data['locked_out']
            now = self._get_now()
            dtime = locked_out - now
            if dtime.seconds < TIMEOUT:
                reconnect_in = dtime.seconds

        response_data.update({
            'is_locked': is_locked,
            'locked_by': locked_by,
            'reconnected': reconnected,
            'reconnect_in': reconnect_in,
        })

        return response_data



from __future__ import unicode_literals

from django.db import models
from django.utils import timezone
from admin_page_lock.models.base_model import BasePageLockModel
from admin_page_lock.settings import (
    KEEP_DB_LOCKS,
    URL_IGNORE_PARAMETERS,
)


class DatabasePageLockModel(BasePageLockModel, models.Model):
    """
    Parameters:
      + active       whether data is still active
      + locked       whether page is locked
      + locked_at    time when page was locked
      + locked_out   time when page will be unlocked
      + locked_by    user_reference of user that locked page
      + parameters   url parameters in JSON
      + url          url of locked page
    """
    url = models.URLField()
    url_parameters = models.CharField(max_length=1024, null=True, blank=True)
    active = models.BooleanField(default=True)
    user_reference = models.CharField(max_length=255, null=True, blank=True)
    locked = models.BooleanField(default=True)
    locked_at = models.DateTimeField(db_index=True)
    locked_out = models.DateTimeField(db_index=True)

    def __unicode__(self):
        return '{}'.format(self.pk)

    @classmethod
    def _get_query_kwargs(cls, page_settings):
        query_kwargs = {
            'active': True,
            'url': page_settings['page_url'],
        }
        if not URL_IGNORE_PARAMETERS:
            query_kwargs['url_parameters'] = page_settings['page_url_parameters']  # noqa: E501

        return query_kwargs

    @classmethod
    def deactivate(cls, page_settings):
        query_kwargs = cls._get_query_kwargs(page_settings)
        page_locks = cls.objects.filter(**query_kwargs)
        if KEEP_DB_LOCKS:
            page_locks.update(active=False)
        else:
            page_locks.delete()

    @classmethod
    def get_data(cls, page_settings):
        query_kwargs = cls._get_query_kwargs(page_settings)
        page_locks = cls.objects.filter(**query_kwargs)

        # Filter out data with `locked_out` older then now.
        page_locks = page_locks.filter(locked_out__gt=timezone.now())

        # Get the latest instance of `DatabasePageLockModel` and check
        # its existence.
        page_lock = page_locks.first()
        if (
            page_lock is None or
            not isinstance(page_lock, cls)
        ):
            return None

        return {
            'locked_at': page_lock.locked_at,
            'locked_out': page_lock.locked_out,
            'user_reference': page_lock.user_reference
        }

    def save(self, *args, **kwargs):
        # Deactive current instance with `locked_out` older then now.
        if self.locked_out < timezone.now():
            self.active = False

        super(DatabasePageLockModel, self).save(*args, **kwargs)

    @classmethod
    def set_data(cls, page_settings, data):
        data['url'] = page_settings['page_url']
        if not URL_IGNORE_PARAMETERS:
            data['url_parameters'] = page_settings['page_url_parameters']

        try:
            cls.objects.create(**data)
        except cls.DoesNotExist:
            raise

    class Meta:
        ordering = ('locked_at',)
        app_label = 'admin_page_lock'
        verbose_name = 'Page Lock'
        verbose_name_plural = 'Page Locks'



from __future__ import unicode_literals

from admin_page_lock.mixins.base_mixin import BaseLockingMixin
from admin_page_lock.settings import DISABLE


class PageLockViewMixin(BaseLockingMixin):
    def dispatch(self, req, *args, **kwargs):
        self.req = req
        return super(PageLockViewMixin, self).dispatch(req, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(PageLockViewMixin, self).get_context_data(**kwargs)

        if not DISABLE:
            result = self._open_page_connection_data(self.req)
            context.update(self._add_extra_content(self.req, result))

        return context
